import { Location } from 'vscode-languageserver';
import { FindComponentReferencesProvider } from '../../interfaces';
import { LSAndTSDocResolver } from '../LSAndTSDocResolver';
export declare class FindComponentReferencesProviderImpl implements FindComponentReferencesProvider {
    private readonly lsAndTsDocResolver;
    constructor(lsAndTsDocResolver: LSAndTSDocResolver);
    findComponentReferences(uri: string): Promise<Location[] | null>;
    private isEndTag;
}
