"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.foo = foo;
/**
 * bars
 * @author John
 */
function foo() { return false; }
//# sourceMappingURL=documentation.js.map