import { DocumentHighlight, Position } from 'vscode-languageserver-types';
import { Document, TagInformation } from '../documents';
export declare function wordHighlightForTag(document: Document, position: Position, tag: TagInformation | null, wordPattern: RegExp): DocumentHighlight[] | null;
