export declare class Logger {
    private static logErrorsOnly;
    private static logDebug;
    static setLogErrorsOnly(logErrorsOnly: boolean): void;
    static setDebug(debug: boolean): void;
    static log(...args: any): void;
    static error(...args: any): void;
    static debug(...args: any): void;
}
