import ts from 'typescript';
/**
 * This should only be accessed by TS svelte module resolution.
 */
export declare function createSvelteSys(tsSystem: ts.System): ts.System & {
    deleteFromCache: (path: string) => void;
    svelteFileExists: (path: string) => boolean;
    getRealSveltePathIfExists: (path: string) => string;
};
