const { defu, createDefu, defuFn, defuArrayFn } = require("../dist/defu.cjs");

module.exports = defu;

module.exports.defu = defu;
module.exports.default = defu;

module.exports.createDefu = createDefu;
module.exports.defuFn = defuFn;
module.exports.defuArrayFn = defuArrayFn;
