import { Location } from 'vscode-languageserver';
import { FileReferencesProvider } from '../../interfaces';
import { LSAndTSDocResolver } from '../LSAndTSDocResolver';
export declare class FindFileReferencesProviderImpl implements FileReferencesProvider {
    private readonly lsAndTsDocResolver;
    constructor(lsAndTsDocResolver: LSAndTSDocResolver);
    fileReferences(uri: string): Promise<Location[] | null>;
    private getSnapshotForPath;
}
