import { Position } from 'vscode-languageserver';
import { AttributeContext } from '../../../lib/documents/parseHtml';
import { SvelteDocument } from '../SvelteDocument';
export declare function attributeCanHaveEventModifier(attributeContext: AttributeContext): boolean;
export declare function inStyleOrScript(svelteDoc: SvelteDocument, position: Position): boolean;
