import type tsAst from '@typescript/native-preview/unstable/ast' with { 'resolution-mode': 'import' };
import type tsApiSync from '@typescript/native-preview/unstable/sync' with { 'resolution-mode': 'import' };
export { tsAst, tsApiSync };
