import { Position, DocumentHighlight } from 'vscode-languageserver-protocol';
import { Document } from '../../../lib/documents';
import { DocumentHighlightProvider } from '../../interfaces';
import { LSAndTSDocResolver } from '../LSAndTSDocResolver';
export declare class DocumentHighlightProviderImpl implements DocumentHighlightProvider {
    private readonly lsAndTsDocResolver;
    constructor(lsAndTsDocResolver: LSAndTSDocResolver);
    findDocumentHighlight(document: Document, position: Position): Promise<DocumentHighlight[] | null>;
    private convertHighlightKind;
    private getSvelteDocumentHighlight;
    private findCandidateSvelteTag;
    private getTagHighlight;
    private getBlockHighlight;
    private getElseHighlightsForIfBlock;
    private getAwaitBlockHighlight;
    private notInGeneratedCode;
}
