import { Position, Location } from 'vscode-languageserver-protocol';
import { Document } from '../../../lib/documents';
import { TypeDefinitionProvider } from '../../interfaces';
import { LSAndTSDocResolver } from '../LSAndTSDocResolver';
export declare class TypeDefinitionProviderImpl implements TypeDefinitionProvider {
    private readonly lsAndTsDocResolver;
    constructor(lsAndTsDocResolver: LSAndTSDocResolver);
    getTypeDefinition(document: Document, position: Position): Promise<Location[] | null>;
}
