import { TraceMap } from '@jridgewell/trace-mapping';
import { Position } from 'vscode-languageserver';
import { SourceMapDocumentMapper } from '../../lib/documents';
export declare class ConsumerDocumentMapper extends SourceMapDocumentMapper {
    private nrPrependesLines;
    constructor(traceMap: TraceMap, sourceUri: string, nrPrependesLines: number);
    getOriginalPosition(generatedPosition: Position): Position;
    getGeneratedPosition(originalPosition: Position): Position;
    isInGenerated(): boolean;
}
