import { FoldingRange } from 'vscode-languageserver-types';
import { Document } from '../../lib/documents';
export declare function getFoldingRanges(document: Document, capability?: {
    lineFoldingOnly?: boolean;
}): FoldingRange[];
