export interface GlobalVar {
    name: string;
    filename: string;
    value: string;
}
export declare class GlobalVars {
    private fsWatcher?;
    private watchedFiles;
    private globalVars;
    private readonly workspaceRoot;
    constructor(workspaceRoot: string[]);
    watchFiles(filesToWatch: string): void;
    private createIgnoreMatcher;
    private updateForFile;
    getGlobalVars(): GlobalVar[];
}
