import { LanguageService, LanguageServiceOptions } from 'vscode-css-languageservice';
export declare function getLanguage(kind?: string): "css" | "scss" | "less";
export type CSSLanguageServices = Record<'css' | 'less' | 'scss', LanguageService>;
export declare function getLanguageService(langs: CSSLanguageServices, kind?: string): LanguageService;
export declare function createLanguageServices(options?: LanguageServiceOptions): CSSLanguageServices;
