import ts from 'typescript';
import { CompletionList } from 'vscode-languageserver';
import { SvelteDocumentSnapshot } from '../DocumentSnapshot';
export declare function getJsDocTemplateCompletion(snapshot: SvelteDocumentSnapshot, lang: ts.LanguageService, filePath: string, offset: number): CompletionList | null;
