import { FileStat, FileSystemProvider as CSSFileSystemProvider, FileType } from 'vscode-css-languageservice';
import { FileService } from '@vscode/emmet-helper';
import { URI } from 'vscode-uri';
export declare class FileSystemProvider implements CSSFileSystemProvider, FileService {
    constructor();
    stat(uri: URI): Promise<FileStat>;
    stat(uri: string): Promise<FileStat>;
    private unknownStat;
    private getFileType;
    readDirectory(uri: string): Promise<Array<[string, FileType]>>;
    readFile(uri: URI): Promise<Uint8Array>;
}
