"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pseudoClass = void 0;
exports.pseudoClass = [
    {
        name: ':global()',
        description: `[svelte] :global modifier

Applying styles to a selector globally`,
        references: [
            {
                name: 'Svelte.dev Reference',
                url: 'https://svelte.dev/docs/svelte/global-styles'
            }
        ]
    }
];
//# sourceMappingURL=svelte-selectors.js.map