"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDefaultSomeType = getDefaultSomeType;
function getDefaultSomeType() {
    return {
        hi: 'foo'
    };
}
//# sourceMappingURL=some-type.js.map