"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComponentDef6 = exports.ComponentDef3 = exports.ComponentDef2 = exports.ComponentDef = void 0;
const SvelteComponentTyped = null;
class ComponentDef extends SvelteComponentTyped {
}
exports.ComponentDef = ComponentDef;
class ComponentDef2 extends SvelteComponentTyped {
}
exports.ComponentDef2 = ComponentDef2;
class ComponentDef3 extends SvelteComponentTyped {
}
exports.ComponentDef3 = ComponentDef3;
exports.ComponentDef6 = ComponentDef3;
class ComponentDef3_ext extends SvelteComponentTyped {
}
//# sourceMappingURL=ComponentDef.js.map