"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.store = exports.ScndImport = void 0;
class ScndImport {
}
exports.ScndImport = ScndImport;
exports.store = null;
//# sourceMappingURL=to-import.js.map