"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.load = void 0;
// In a real SvelteKit application, $types are autogenerated, for this test we manually create them to our liking
// @ts-expect-error - to silence tsc
exports.load = decorator((event) => event.test);
// Dummy decorator function, to open the door to SvelteKit extensions
function decorator(fn) {
    return fn;
}
//# sourceMappingURL=+page.js.map