import { DidChangeWatchedFilesParams, RelativePattern } from 'vscode-languageserver';
type DidChangeHandler = (para: DidChangeWatchedFilesParams) => void;
export declare class FallbackWatcher {
    private readonly watcher;
    private readonly callbacks;
    private undeliveredFileEvents;
    constructor(watchExtensions: string[], workspacePaths: string[]);
    private convert;
    private onFSEvent;
    private readonly scheduleTrigger;
    onDidChangeWatchedFiles(callback: DidChangeHandler): void;
    watchDirectory(patterns: RelativePattern[]): void;
    dispose(): void;
}
export {};
