"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hi = hi;
/**
 *
 * @param cb callback because if the module resolution failed there will be a noImplicitAny error
 */
function hi(cb) { }
//# sourceMappingURL=imported.js.map