import { TextDocumentEdit, WorkspaceEdit } from 'vscode-languageserver';
import { FileRename, UpdateImportsProvider } from '../../interfaces';
import { LSAndTSDocResolver } from '../LSAndTSDocResolver';
import { LanguageServiceContainer } from '../service';
export declare class UpdateImportsProviderImpl implements UpdateImportsProvider {
    private readonly lsAndTsDocResolver;
    constructor(lsAndTsDocResolver: LSAndTSDocResolver, useCaseSensitiveFileNames: boolean);
    private getCanonicalFileName;
    updateImports(fileRename: FileRename): Promise<WorkspaceEdit | null>;
    updateImportForSingleService(oldPath: string, newPath: string, lsContainer: LanguageServiceContainer, documentChanges: Map<string, TextDocumentEdit>): Promise<void>;
}
