"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatDate = formatDate;
exports.formatDate2 = formatDate2;
function formatDate(date) { }
function formatDate2(date) {
    formatDate(date instanceof Date ? date : parseDate(date));
}
function parseDate(date) {
    return new Date(date);
}
//# sourceMappingURL=util.js.map