"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.load = load;
// In a real SvelteKit application, $types are autogenerated, for this test we manually create them to our liking
// @ts-expect-error - to silence tsc
function load(event) {
    return event.test;
}
//# sourceMappingURL=+page.js.map