"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SomeImportedClass = void 0;
class SomeImportedClass {
}
exports.SomeImportedClass = SomeImportedClass;
//# sourceMappingURL=some-class.js.map